#include "DataFile.h"

DataFile::DataFile(char *fname)
{
  //showSomeInfo("Initializing the DataFile object");
  strcpy(name, fname);
};

DataFile::~DataFile()
{
  FreeMem();
}

void DataFile::AllocMem()
{
  //showSomeInfo("Alloctating the memory for DataFile");
  NewMatrix(data, dimy, dimx, ELEV);
}


void DataFile::FreeMem()
{
  DeleteMatrix(data);
}

void PGMFile::Read()
{
  //showSomeInfo("Reading the data file", -1);

  int n, i, j;
  char buf[255];
  char c;

  FILE *f = fopen(name, "rb");
  if (f==NULL) fatal("Cannot open the data file!");
  // the pgm header -- P5 or sth
  fscanf(f, "%s", &buf);
  if ((c=fgetc(f))=='#') { //comment
    for(; (c=fgetc(f))!='\n'; ) ;
  } else ungetc(c, f);
  // the dimensions: number of columns and the rows, respectively
  fscanf(f, "%d %d", &dimx, &dimy);
  if ((c=fgetc(f))=='#') { //comment
    for(; (c=fgetc(f))!='\n'; ) ;
  } else ungetc(c, f);
  // the maximum value -- determines the data element size
  fscanf(f, "%d", &n);
  if (n!=255) 
    fatal("Can only handle 255-shades files");
  // and the last carridge return character
  fgetc(f); 

  // prepare the necessary memory space 
  AllocMem();

  // and read the actual data
  for(i=0; i<dimy; i++)
    for(j=0; j<dimx; j++) 
      if((data[i][j]=fgetc(f))==EOF)
	fatal("File too short: ", name);
    
  // just to make sure the file is valid
  if (fgetc(f)!=EOF)
    fatal("File too long: ", name);
  fclose(f);
}


void TableFile::Read()
{
  //showSomeInfo("Reading the data file", -1);

  int n, i, j;
  dimx = dimy = 1000;

  // prepare the necessary memory space 
  AllocMem();

  // and read the actual data
  FILE *f = fopen(name, "rb");
  if (f==NULL) fatal("Cannot open the data file!");
  for(i=0; i<dimy; i++)
    for(j=0; j<dimx; j++) {
      fscanf(f, "%d", &n);
      fscanf(f, "%d", &n);
      fscanf(f, "%f", &(data[i][j]));
      fscanf(f, "%d", &n);
      fscanf(f, "%d", &n);
      fscanf(f, "%d", &n);
      fscanf(f, "%d", &n);
      fscanf(f, "%d", &n);
    }
  //  just to make sure the file is valid
  if (fgetc(f)!=EOF)
    fatal("File too long: ", name);
  fclose(f);
}

void URLFile::Read()
{
#ifdef DEBUG
  printf("URLFile::URL is %s\n", name);
#endif
  WebNode *node = new WebNode(name);

  char *buf = node->getBuffer();
  dimx = ((buf[0]*256 + buf[1])*256 + buf[2])*256 + buf[3];
  dimy = ((buf[4]*256 + buf[5])*256 + buf[6])*256 + buf[7];

  buf = buf + 8;

  AllocMem();

  int i, j;
  for(j=0; j<dimy; j++) 
    for(i=0; i<dimx; i++)
      {
	data[dimy-j-1][i]=buf[2*(j*dimx+i)]*256+buf[2*(j*dimx+i)+1];
	//data[j][i]=buf[2*(j*dimx+i)]*256+buf[2*(j*dimx+i)+1];
      }
  delete node->getBuffer();
}


void 
URLCompressedDEM::Read()
{
  WebNode *node = new WebNode(name);

  char *buf = node->getBuffer();
  dimx = buf[0]*256 + buf[1];
  dimy = dimx;
  
  AllocMem();
  
  buf = buf + 2;

  wv_t *wv = new_wv(dimx);
  int streamLength = node->getBufSize();
  
  dec_elev_wv(wv,(unsigned char*)buf,streamLength,data);
  
  delete node->getBuffer();
  delete_wv(&wv);
}

